package lab.game;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import javafx.geometry.Dimension2D;
import javafx.geometry.Point2D;
import javafx.geometry.Rectangle2D;
import javafx.scene.canvas.GraphicsContext;
import javafx.scene.image.Image;
import lab.Setup;

public class LochNess extends WorldEntity implements Collisionable {

	private static Logger log = LogManager.getLogger(LochNess.class);

	private static final Random RANDOM = new Random();

	private Point2D speed;
	private Image image;
	private List<DeadListener> deadListeners = new ArrayList<>();

	public LochNess(Scene scene) {
		super(scene, new Point2D(0, 0), 90);
		this.scene = scene;
		image = new Image(LochNess.class.getResourceAsStream("LochNess.gif"));
		position = new Point2D(
				RANDOM.nextDouble(scene.getSize().getWidth() * Setup.getInstance().getLochnessMinXPopsition(),
						scene.getSize().getWidth()),
				RANDOM.nextDouble(scene.getSize().getHeight() * Setup.getInstance().getLochnessMinYPopsition(),
						scene.getSize().getHeight() - image.getHeight()));
		speed = new Point2D(-RANDOM.nextDouble(Setup.getInstance().getLochnessMinSpeed(),
				Setup.getInstance().getLochnessMaxSpeed()), 0);
	}

	@Override
	public void drawInternal(GraphicsContext gc) {
		gc.drawImage(image, position.getX(), position.getY());
		gc.strokeRect(position.getX(), position.getY(), image.getWidth(), image.getHeight());
	}

	@Override
	public void simulate(double deltaTime) {
		position = position.add(speed.multiply(deltaTime / 1_000_000_000));
		if (position.getX() + image.getWidth() < 0) {
			position = new Point2D(scene.getSize().getWidth(), position.getY());
		}
		if (position.getX() > scene.getSize().getWidth() + 5) {
			position = new Point2D(scene.getSize().getWidth(), position.getY());
			speed = speed.multiply(-1);
		}
		log.trace("LochNess position: {}", position);
	}

	@Override
	public Rectangle2D getBoundingBox() {
		return new Rectangle2D(position.getX(), position.getY(), image.getWidth(), image.getHeight());
	}

	public void changeDirection() {
		speed = speed.multiply(-1);
		log.debug("LochNess chaned direction.");
	}

	@Override
	public boolean intersect(Rectangle2D another) {
		return getBoundingBox().intersects(another);
	}

	@Override
	public void hitBy(Collisionable another) {
		log.trace("LochNess hitted by {}.", another);
		if (another instanceof Boat) {
			scene.remove(this);
			scene.add(new Rock(scene, getPosition(), new Dimension2D(10, 10)));
			fireLochNessDead();
		}

	}

	public boolean addDeadListener(DeadListener e) {
		return deadListeners.add(e);
	}

	public boolean removeDeadListener(DeadListener o) {
		return deadListeners.remove(o);
	}

	private void fireLochNessDead() {
		for (DeadListener deadListener : deadListeners) {
			deadListener.lochnessDead();
		}
	}
}
